------------------------------------------------------------
--                                                        --
-- FSAL sample demonstration                              --
--                                                        --
-- Copyright (C) 2024 Michiyo Tagami                      --
--                                                        --
-- Released under the MOUSEY license                      --
--                                                        --
------------------------------------------------------------

require("fullscreen")

function anyInitialization()
  math.randomseed(os.time())
  love.window.setMode(800, 600, {resizable=true})
  love.window.setTitle("FSAL EXAMPLE")
  fsbg = love.graphics.newCanvas(200, 200)
  love.graphics.setCanvas(fsbg)
  for tmpI=0,3000 do
    love.graphics.setColor(math.random(0,255)/255, math.random(0,64)/255, 0)
    love.graphics.points({math.random(0,299), math.random(0,299)})
  end
  love.graphics.setCanvas()
  mx, my = 0, 0; cx, cy = -100, -100; at = 0
  collist = {}; ct = 0; traces = {}; tt = 0
  for tmpI=1,30 do
    collist[tmpI] = {0, 0, math.abs(tmpI-15)*2/30}
    traces[tmpI] = {-100, -100}
    --print(">", tmpI, traces[tmpI])
  end
  mf = love.graphics.newFont(15)
  bf = love.graphics.newFont(20)
end

function anyUpdate(dt, tx, ty)
  at = math.max(at - dt, 0)
  if (at==0) then
    cx, cy = -100, -100
    --print("eee")
  end
  ct = ct + dt
  if (ct>=0.1) then
    local tmpC = collist[30]
    for tmpI = 30,2,-1 do
      collist[tmpI] = collist[tmpI-1]
    end
    collist[1] = tmpC
    ct = ct - 0.1
  end
  tt = tt + dt
  if (tt>=0.1) then
    for tmpI=30,2,-1 do
      traces[tmpI] = traces[tmpI-1]
      --print("=", tmpI, traces[tmpI])
    end
    traces[1] = {tx, ty}
    tt = tt - 0.02
  end
end

function drawAnything()
  love.graphics.translate(400, 300)
  for tmpR=30,1,-1 do
    --love.graphics.setColor(math.random(100)/100, math.random(100)/100, math.random(100)/100)
    love.graphics.setColor(collist[tmpR])
    love.graphics.circle("fill", 0, 0, tmpR*10)
  end
  love.graphics.origin()
  for tmpI=30,2,-1 do
    --print(tmpI, traces[tmpI])
    love.graphics.setColor(1, 1, 0, (30-tmpI)/30)
    love.graphics.circle("line", traces[tmpI][1], traces[tmpI][2], 10-(tmpI/3))
  end
  love.graphics.setColor(0, 1, 0, 0.5)
  --print(cx,cy)
  love.graphics.circle("fill", cx, cy, 10)
  love.graphics.setColor(0, 1, 1)
  love.graphics.circle("line", mx, my, 10)
  love.graphics.setColor(1, 0.5, 0)
  love.graphics.setFont(bf)
  love.graphics.printf("("..mx..","..my..")", 0, 290, 800, "center")
  love.graphics.setColor(1, 1, 1)
  love.graphics.setFont(mf)
  love.graphics.printf("F11: Toggle Fullscreen/Window mode.", 0, 555, 800, "right")
  love.graphics.printf("F10: Turn ON/OFF mouse area clipping.", 0, 570, 800, "right")
  love.graphics.printf("F11: Reset window size.", 0, 585, 800, "right")
end

function love.load()
  anyInitialization()
  fsal_initialize(800, 600, false)
  canvas = fsal_canvas()
  mouseclip = false
end

function love.update(dt)
  local tx, ty = fsal_clipmouse()
  anyUpdate(dt, tx, ty)
end

function love.draw()
  fsal_drawbgimage(fsbg)
  fsal_drawstart(canvas)
  drawAnything()
  fsal_drawend()
  fsal_show(canvas)
end

function love.keypressed(key)
  if (key=="f11") then
    local tmpS = not fsal_getscmode()
    fsal_modechange(tmpS, tmpS)
  elseif (key=="f12") then
    fsal_resetwinsize()
  elseif (key=="f10") then
    mouseclip = not mouseclip
    fsal_setclipmouse(mouseclip, mouseclip)
  end
end

function love.resize(w, h)
  fsal_resize(w, h)
end

function love.mousemoved(x, y)
  mx, my = fsal_getvirtualmouse(x, y)
end

function love.mousepressed(x, y)
  cx, cy = fsal_getvirtualmouse(x, y)
  at = 0.5
end
